'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getConfig = exports.initConfig = exports.getAppRootPath = exports.setUserConfig = exports.getEntryFilePath = exports.error = exports.isSingleAppMode = exports.templateReplace = exports.relativePath = exports.translateEs6to5 = exports.checkFileNameValid = exports.checkFileDuplicate = undefined;

var _babelCore = require('babel-core');

var babel = _interopRequireWildcard(_babelCore);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

_colors2.default.setTheme({
  silly: 'rainbow',
  input: 'grey',
  verbose: 'cyan',
  prompt: 'red',
  info: 'green',
  data: 'blue',
  help: 'cyan',
  warn: 'yellow',
  debug: 'magenta',
  error: 'red'
});

var userConfig = {};

function setUserConfig(conf) {
  userConfig = conf;
}

function initConfig(conf) {
  userConfig = conf;

  userConfig.srcFolder = conf.src || './src';
  userConfig.componentsFolder = conf.components || conf.src + "/components";
  userConfig.singleApp = isSingleAppMode(conf.src);
}

function getConfig() {
  return userConfig;
}

/***
 * 检测fileList中后缀为fileType的file文件是否重复 如果重复则给出提示并退出node
 * @param fileList 文件路径列表
 * @param file 文件名
 * @param fileType 文件后缀
 */
function checkFileDuplicate(fileList, file, fileType) {
  var files = _lodash2.default.filter(fileList, function (item) {
    return _lodash2.default.endsWith(item, '/' + file + '.' + fileType);
  });

  if (files.length > 1) {
    console.error(_colors2.default.red(file + '.' + fileType + '\u6587\u4EF6\u547D\u540D\u91CD\u590D, \u540C\u4E00\u4E2A\u5E94\u7528\u4E0B' + fileType + '\u7C7B\u578B\u7684\u6587\u4EF6\u547D\u540D\u4E0D\u80FD\u91CD\u590D\uFF0C\u547D\u540D\u91CD\u590D\u6587\u4EF6\uFF1A\n' + files.join('\n')));
    process.exit();
  }
}

/***
 * 检测vue和vuex文件命名是否规范
 * @param filename 文件名
 * @param fileType 文件后缀
 */
function checkFileNameValid(filename, fileType) {
  if (filename.indexOf('-') > 0 || filename.indexOf('.') > 0) {
    console.error(_colors2.default.red('\u6587\u4EF6\u540D\u8BF7\u4F7F\u7528\u9A7C\u5CF0\u5F0F\u547D\u540D, \u5982myNameIsWisedu\uFF01\u547D\u540D\u9519\u8BEF\u6587\u4EF6\uFF1A' + filename + '.' + fileType));
    process.exit();
  }
}

/***
 * 讲es6写的代码转换成es5
 * @param file
 */
function translateEs6to5(file) {
  var content = _fs2.default.readFileSync(file);
  var result = babel.transform(content, {
    presets: ['env']
  });
  var exports = {};
  eval(result.code);

  return exports;
}

/***
 * 获取绝对文件路径相对于tempfiles的相对路径
 * @param filePath 绝对文件路径
 */
function relativePath(filePath) {
  return _path2.default.relative(__dirname + '/tempfiles', filePath);
}

/**
 * 模板替换方法
 * @param  {[type]} template [模板]
 * @param  {[type]} config   [配置对象]
 * @return {[type]}          [description]
 */
function templateReplace(template, config) {
  Object.keys(config).forEach(function (item) {
    var re = new RegExp('\\{\\{' + item + '\\}\\}', 'g');
    var statementre = new RegExp('\\\'\\{\\{' + item + '\\}\\}\\\'', 'g');

    if (config[item].statement) {
      template = template.replace(statementre, config[item].content);
      return;
    }
    if (!config[item].relativePath) {
      template = template.replace(re, config[item].content);
      return;
    }

    if (_fs2.default.existsSync(config[item].content)) {
      template = template.replace(re, relativePath(config[item].content)).replace(/\\/g, '/');
    } else {
      if (config[item].required) {
        console.error(_colors2.default.red(config[item].content + '文件不存在!'));
        process.exit();
      } else {
        template = template.replace(re, config[item].default);
      }
    }
  });

  return template;
}

/***
 * 检查项目类型 单app或多app模式
 */
function isSingleAppMode(config) {
  if (!userConfig.srcFolder) {
    initConfig(config);
  }
  var projectType = null;
  var appsPath = _path2.default.resolve(userConfig.srcFolder) + '/apps';

  try {
    var stat = _fs2.default.lstatSync(appsPath);

    if (!stat.isDirectory()) {
      return true;
    } else {
      return false;
    }
  } catch (err) {
    return true;
  }
}

function error(message) {
  console.error(_colors2.default.red(message));
  process.exit();
}

function getEntryFilePath(config) {
  if (!userConfig.srcFolder) {
    initConfig(config);
  }
  return __dirname + '/tempfiles';
}

function getAppRootPath(config) {
  if (!userConfig.srcFolder) {
    initConfig(config);
  }
  var singleAppMode = isSingleAppMode();

  return singleAppMode ? _path2.default.resolve(userConfig.srcFolder) : _path2.default.resolve(userConfig.srcFolder + '/apps');
}

exports.checkFileDuplicate = checkFileDuplicate;
exports.checkFileNameValid = checkFileNameValid;
exports.translateEs6to5 = translateEs6to5;
exports.relativePath = relativePath;
exports.templateReplace = templateReplace;
exports.isSingleAppMode = isSingleAppMode;
exports.error = error;
exports.getEntryFilePath = getEntryFilePath;
exports.setUserConfig = setUserConfig;
exports.getAppRootPath = getAppRootPath;
exports.initConfig = initConfig;
exports.getConfig = getConfig;